package com.bocloud.pms.booter.config;

import com.bocloud.common.utils.MapTools;
import org.springframework.amqp.core.Binding;
import org.springframework.amqp.core.BindingBuilder;
import org.springframework.amqp.core.DirectExchange;
import org.springframework.amqp.core.Queue;
import org.springframework.amqp.rabbit.connection.ConnectionFactory;
import org.springframework.amqp.rabbit.core.RabbitAdmin;
import org.springframework.amqp.rabbit.core.RabbitTemplate;
import org.springframework.amqp.support.converter.Jackson2JsonMessageConverter;
import org.springframework.amqp.support.converter.MessageConverter;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.config.ConfigurableBeanFactory;
import org.springframework.boot.context.properties.ConfigurationProperties;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.context.annotation.Scope;

import java.util.Map;

/**
 * rabbitmq配置类
 *
 * @author wangyu
 * @version 1.0
 * @since 2018年3月30日
 */
@Configuration
@ConfigurationProperties(prefix = "spring.rabbitmq")
public class RabbitMQConfiguration {

	@Autowired
	private ConnectionFactory connectionFactory;
	private Map<String, Object> arguments = MapTools.simpleMap("x-ha-policy", "all");

	/**
	 * 声明资源申请流程队列
	 *
	 * @return
	 */
	@Bean
	public Queue declareTaskQueue() {
		return new Queue("task.result.queue", true, false, false, arguments);
	}

	/**
	 * 声明资源编排节点请求队列
	 *
	 * @return
	 */
	@Bean
	public Queue declareRosQueue() {
		return new Queue("bocloud.ros.queue", true, false, false, arguments);
	}

	@Bean
	public Queue declareFlowResultQueue() {
		return new Queue("flow.execute.result.queue", true, false, false, arguments);
	}

	/**
	 * 声明资源订购队列
	 *
	 * @return
	 */
	@Bean
	public Queue declareCreationQueue() {
		return new Queue("paas.resource.creation.queue", true, false, false, arguments);
	}

	/**
	 * 声明资源变更队列
	 *
	 * @return
	 */
	@Bean
	public Queue declareModificationQueue() {
		return new Queue("paas.resource.modification.queue", true, false, false, arguments);
	}

	/**
	 * 声明资源退订队列
	 *
	 * @return
	 */
	@Bean
	public Queue declareDeletionQueue() {
		return new Queue("paas.resource.deletion.queue", true, false, false, arguments);
	}

	/**
	 * 声明资源延期队列
	 *
	 * @return
	 */
	@Bean
	public Queue declareExtensionQueue() {
		return new Queue("paas.resource.extension.queue", true, false, false, arguments);
	}

	/**
	 * 声明集群节点变更队列
	 *
	 * @return
	 */
	@Bean
	public Queue declareNodeAlterationQueue() {
		return new Queue("software.node.event", true, false, false, arguments);
	}

	/**
	 * 声明软件事件队列
	 *
	 * @return
	 */
	@Bean
	public Queue declareSoftEventQueue() {
		return new Queue("software.event.queue", true, false, false, arguments);
	}

	/**
	 * 声明任务服务结果请求队列
	 *
	 * @return
	 */
	@Bean
	public Queue declareTaskResultQueue() {
		return new Queue("service.task.result", true, false, false, arguments);
	}

	/**
	 * 声明软件安装消息队列
	 *
	 * @return
	 */
	@Bean
	public Queue declareSoftInstallQueue() {
		return new Queue("software.install.queue", true, false, false, arguments);
	}

	/**
	 * 声明默认的交换
	 *
	 * @return
	 */
	@Bean
	public DirectExchange deafultExchange() {
		return new DirectExchange("bocloud.direct.exchange", true, true);
	}

	/**
	 * 资源软件安装消息队列绑定exchange
	 *
	 * @return
	 */
	@Bean
	public Binding bindingSoftInstallQueue() {
		return BindingBuilder.bind(declareSoftInstallQueue()).to(deafultExchange()).withQueueName();
	}

	/**
	 * 资源任务服务结果请求队列绑定exchange
	 *
	 * @return
	 */
	@Bean
	public Binding bindingTaskResultQueue() {
		return BindingBuilder.bind(declareTaskResultQueue()).to(deafultExchange()).withQueueName();
	}

	/**
	 * 软件事件列绑定exchange
	 *
	 * @return
	 */
	@Bean
	public Binding bindingRdsEventQueue() {
		return BindingBuilder.bind(declareSoftEventQueue()).to(deafultExchange()).withQueueName();
	}

	/**
	 * 绑定资源订购队列
	 *
	 * @return
	 */
	@Bean
	public Binding bindingCreationQueue() {
		return BindingBuilder.bind(declareCreationQueue()).to(deafultExchange()).withQueueName();
	}

	/**
	 * 绑定资源变更队列
	 *
	 * @return
	 */
	@Bean
	public Binding bindingModificationQueue() {
		return BindingBuilder.bind(declareModificationQueue()).to(deafultExchange()).withQueueName();
	}

	/**
	 * 绑定资源退订队列
	 *
	 * @return
	 */
	@Bean
	public Binding bindingDeletionQueue() {
		return BindingBuilder.bind(declareDeletionQueue()).to(deafultExchange()).withQueueName();
	}

	/**
	 * 绑定资源延期队列
	 *
	 * @return
	 */
	@Bean
	public Binding bindingExtensionQueue() {
		return BindingBuilder.bind(declareExtensionQueue()).to(deafultExchange()).withQueueName();
	}

	/**
	 * 绑定集群节点变更队列
	 *
	 * @return
	 */
	@Bean
	public Binding bindingNodeAlterationQueue() {
		return BindingBuilder.bind(declareNodeAlterationQueue()).to(deafultExchange()).withQueueName();
	}

	/**
	 * 绑定作业结果队列
	 *
	 * @return
	 */
	@Bean
	public Binding bindingTaskQueue() {
		return BindingBuilder.bind(declareTaskQueue()).to(deafultExchange()).withQueueName();
	}

	/**
	 * 资源编排节点请求队列绑定exchange
	 *
	 * @return
	 */
	@Bean
	public Binding bindingRosQueue() {
		return BindingBuilder.bind(declareRosQueue()).to(deafultExchange()).withQueueName();
	}

	@Bean
	public Binding bindingFlowResultQueue() {
		return BindingBuilder.bind(declareFlowResultQueue()).to(deafultExchange()).withQueueName();
	}

	@Bean
	public MessageConverter messageConverter() {
		return new Jackson2JsonMessageConverter();
	}

	@Bean
	@Scope(ConfigurableBeanFactory.SCOPE_PROTOTYPE)
	public RabbitTemplate amqpTemplate() {
		RabbitTemplate amqpTemplate = new RabbitTemplate(connectionFactory);
		amqpTemplate.setMessageConverter(messageConverter());
		amqpTemplate.setExchange(deafultExchange().getName());
		return amqpTemplate;
	}

	@Bean
	@Scope(ConfigurableBeanFactory.SCOPE_PROTOTYPE)
	public RabbitAdmin rabbitAdmin() {
		return new RabbitAdmin(connectionFactory);
	}
}
