package com.bocloud.pms.service;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.common.model.*;
import com.bocloud.common.utils.Common;
import com.bocloud.common.utils.GridHelper;
import com.bocloud.common.utils.MapTools;
import com.bocloud.pms.entity.Cms;
import com.bocloud.pms.entity.CmsLog;
import com.bocloud.pms.entity.CmsNode;
import com.bocloud.pms.entity.CmsTopic;
import com.bocloud.pms.entity.resource.enums.ResourceCategory;
import com.bocloud.pms.interfaces.middleware.CmsService;
import com.bocloud.pms.interfaces.middleware.CmsTopicService;
import com.bocloud.pms.repository.CmsNodeRepository;
import com.bocloud.pms.repository.CmsRepository;
import com.bocloud.pms.repository.CmsTopicRepository;
import com.bocloud.pms.service.utils.ResourceEventPublisher;
import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import org.apache.kafka.clients.admin.*;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.concurrent.ThreadPoolTaskExecutor;
import org.springframework.stereotype.Service;

import java.util.*;

/**
 * Created by liuyuanyuan on 2020/3/12 17:30
 */
@Service("cmsTopicService")
public class CmsTopicServiceImpl implements CmsTopicService {

    private static Logger logger = LoggerFactory.getLogger(CmsTopicServiceImpl.class);
    @Autowired
    private CmsRepository cmsRepository;
    @Autowired
    private CmsTopicRepository cmsTopicRepository;
    @Autowired
    private CmsNodeRepository cmsNodeRepository;
    @Autowired
    private ResourceEventPublisher resourceEventPublisher;
    @Autowired
    private CmsService cmsService;
    @Autowired
    private ThreadPoolTaskExecutor threadPoolTaskExecutor;

    @Override
    public BocloudResult<GridBean<CmsTopic>> list(Long mqId, Pager pager, RequestContext context) {
        try {
            List<Param> params = pager.getParams();
            Integer page = pager.getPage();
            Integer rows = pager.getRows();
            Map<String, String> sorter = pager.getSorter();
            GridBean gridBean = null;
            if (null == params) {
                params = new ArrayList<>();
            }
            if (null == sorter) {
                sorter = Maps.newHashMap();
            }
            sorter.put("gmtCreate", Common.ONE);
            List<CmsTopic> list = cmsTopicRepository.list(mqId, page, rows, params, sorter);
            int total = cmsTopicRepository.count(params);
            gridBean = GridHelper.getBean(page, rows, total, list);
            return new BocloudResult<>(true, gridBean, "查询成功");
        } catch (Exception e) {
            logger.error("list topic failure:", e);
            return new BocloudResult<>(false, "查询失败");
        }
    }

    @Override
    public BocloudResult<CmsTopic> detail(Long mqId, Long topicId) {
        try {
            Cms cms = cmsRepository.query(mqId);
            if (null == cms) {
                logger.warn("cms not exist");
                return new BocloudResult(false, "message queue不存在");
            }
            CmsTopic topic = cmsTopicRepository.query(topicId);
            if (null == topic) {
                logger.warn("topic not exist");
                return new BocloudResult<>(false, "数据不存在");
            }
            return new BocloudResult<>(true, topic, "查询详情成功");
        } catch (Exception e) {
            logger.error("query topic detail fail:", e);
            return new BocloudResult<>(false, "查询详情失败");
        }
    }

    @Override
    public BocloudResult create(CmsTopic mqTopic, RequestContext context, String requestIp) {
        try {
            Cms cms = cmsRepository.query(mqTopic.getMqId());
            if (null == cms) {
                logger.warn("cms not exist");
                return new BocloudResult(false, "message queue不存在");
            }
            if (ResourceCategory.Kafka.name().equalsIgnoreCase(cms.getType())) {
                if (null != cms.getTopics()) {
                    List<Param> params = new ArrayList<>();
                    params.add(new Param(MapTools.simpleMap("mqId", mqTopic.getMqId()), Sign.EQ));
                    int count = cmsTopicRepository.count(params);
                    if (count >= cms.getTopics()) {
                        return new BocloudResult(false, cms.getId(), "topic已达到上限，无法创建");
                    }
                }
                // 启动一个线程从MQ上获取数据
                threadPoolTaskExecutor.submit(() -> {
                    long start = System.currentTimeMillis();
                    BocloudResult result = new BocloudResult(false, "");
                    try {
                        result = execute(cms, mqTopic, "create");
                        if (result.isSuccess()) {
                            mqTopic.setGmtCreate(new Date());
                            mqTopic.setCreatorId(context.getTarget());
                            cmsTopicRepository.save(mqTopic);
                        }
                    } catch (Exception e) {
                        logger.error("create kafka topic failure:", e);
                    }
                    resourceEventPublisher.send(
                            new OperateResult(result.isSuccess(), "创建topic" + (result.isSuccess() ? "成功" : "失败"),
                                    "resource.kafka", null, OperateResult.OperateCategory.ResourceEvent, context));
                    CmsLog log = new CmsLog("消息队列", "topic创建",
                            JSONObject.toJSONString(new BocloudResult(result.isSuccess(),
                                    "topic创建" + (result.isSuccess() ? "成功" : "失败"))),
                            null, requestIp, new Date(), System.currentTimeMillis() - start,
                            cms.getId(), context.getTarget());
                    cmsService.saveLog(log);
                });
                return new BocloudResult(true, "创建topic任务已下发...");
            }
            return new BocloudResult(false, "暂不支持该类操作");
        } catch (Exception e) {
            logger.error("create topic failure:", e);
            return new BocloudResult(false, "创建topic失败");
        }
    }

    @Override
    public BocloudResult remove(Long id, Long topicId, RequestContext context, String requestIp) {
        try {
            Cms cms = cmsRepository.query(id);
            if (null == cms) {
                logger.warn("cms not exist");
                return new BocloudResult(false, "message queue不存在");
            }
            CmsTopic cmsTopic = cmsTopicRepository.query(topicId);
            if (null == cmsTopic) {
                logger.warn("cms topic not exist");
                return new BocloudResult(false, "cms topic不存在");
            }
            if (ResourceCategory.Kafka.name().equalsIgnoreCase(cms.getType())) {
                // 启动一个线程从MQ上获取数据
                threadPoolTaskExecutor.submit(() -> {
                    long start = System.currentTimeMillis();
                    BocloudResult result = new BocloudResult(false, "");
                    try {
                        result = execute(cms, cmsTopic, "remove");
                        if (result.isSuccess()) {
                            cmsTopicRepository.remove(topicId, context.getTarget());
                        }
                    } catch (Exception e) {
                        logger.error("remove kafka topic failure:", e);
                    }
                    resourceEventPublisher.send(
                            new OperateResult(result.isSuccess(), "删除topic" + (result.isSuccess() ? "成功" : "失败"),
                                    "resource.kafka", null, OperateResult.OperateCategory.ResourceEvent, context));
                    CmsLog log = new CmsLog("消息队列", "topic删除",
                            JSONObject.toJSONString(new BocloudResult(result.isSuccess(),
                                    "topic删除" + (result.isSuccess() ? "成功" : "失败"))),
                            null, requestIp, new Date(), System.currentTimeMillis() - start,
                            id, context.getTarget());
                    cmsService.saveLog(log);
                });
                return new BocloudResult(true, "删除topic任务已下发...");
            }
            return new BocloudResult(false, "暂不支持该类操作");
        } catch (Exception e) {
            logger.error("remove kafka topic failure:", e);
            return new BocloudResult(false, "删除topic失败");
        }
    }

    private BocloudResult execute(Cms cms, CmsTopic mqTopic, String action) {
        logger.info("这是第209行。。。。。。。。");
        AdminClient client = null;
        try {
            List<CmsNode> cmsNodes = cmsNodeRepository.list(cms.getId());
            StringBuilder ips = new StringBuilder();
            for (int i = 1; i <= cmsNodes.size(); i++) {
                CmsNode cmsNode = cmsNodes.get(i - 1);
                ips.append(cmsNode.getManageIp()).append(":" + cmsNode.getPort());
                if (i != cmsNodes.size()) {
                    ips.append(",");
                }
            }
            Properties admainProp = new Properties();
            admainProp.put(AdminClientConfig.BOOTSTRAP_SERVERS_CONFIG, ips.toString());
            admainProp.put(AdminClientConfig.REQUEST_TIMEOUT_MS_CONFIG, 60000);
            client = AdminClient.create(admainProp);
            switch (action) {
                case "create":
                    NewTopic newTopic = new NewTopic(mqTopic.getName(), mqTopic.getPartitions().intValue(),
                            mqTopic.getReplicas().shortValue());
                    CreateTopicsOptions options = new CreateTopicsOptions();
                    options.timeoutMs(Integer.valueOf("30000"));
                    CreateTopicsResult result = client.createTopics(Arrays.asList(newTopic), options);
                    while (true) {
                        if (result.all().isDone()) {
                            if (result.all().isCompletedExceptionally()) {
                                logger.error("topic[" + mqTopic.getName() + "]创建失败:" + result.values().get(mqTopic.getName()));
                                return new BocloudResult(false, "topic创建失败");
                            }
                            return new BocloudResult(true, "操作成功");
                        }
                        Thread.sleep(3000);
                    }
                case "edit":
                    Map<String, NewPartitions> newPartitions = Maps.newHashMap();
                    // 创建新的分区的结果
                    newPartitions.put(mqTopic.getName(), NewPartitions.increaseTo(mqTopic.getPartitions().intValue()));
                    CreatePartitionsResult partitions = client.createPartitions(newPartitions);
                    while (true) {
                        if (partitions.all().isDone()) {
                            if (partitions.all().isCompletedExceptionally()) {
                                logger.error("topic[" + mqTopic.getName() + "]新增分区失败:" + partitions.values().get(mqTopic.getName()));
                                return new BocloudResult(false, "新增分区失败");
                            }
                            return new BocloudResult(true, "操作成功");
                        }
                        Thread.sleep(3000);
                    }
                case "remove":
                    DeleteTopicsResult deleteTopicsResult = client.deleteTopics(Lists.newArrayList(mqTopic.getName()));
                    while (true) {
                        if (deleteTopicsResult.all().isDone()) {
                            if (deleteTopicsResult.all().isCompletedExceptionally()) {
                                logger.error("topic[" + mqTopic.getName() + "]topic删除失败:" + deleteTopicsResult.values().get(mqTopic.getName()));
                                return new BocloudResult(false, "topic删除失败");
                            }
                            return new BocloudResult(true, "操作成功");
                        }
                        Thread.sleep(3000);
                    }
                default:
                    return new BocloudResult(false, "暂不支持该类操作");
            }
        } catch (Exception e) {
            logger.info("operate kafka topic error:", e);
            return new BocloudResult(false, "kafka topic操作失败");
        } finally {
            if (null != client) {
                client.close();
            }
        }

    }

    @Override
    public BocloudResult edit(Long id, Long topicId, CmsTopic mqTopic, RequestContext context, String requestIp) {
        try {
            Cms cms = cmsRepository.query(id);
            if (null == cms) {
                logger.warn("cms not exist");
                return new BocloudResult(false, "message queue不存在");
            }
            CmsTopic topic = cmsTopicRepository.query(topicId);
            if (null == topic) {
                logger.warn("topic not exist");
                return new BocloudResult(false, "topic不存在");
            }
            if (ResourceCategory.Kafka.name().equalsIgnoreCase(cms.getType())) {
                // 启动一个线程从MQ上获取数据
                threadPoolTaskExecutor.submit(() -> {
                    long start = System.currentTimeMillis();
                    BocloudResult result = new BocloudResult(false, "");
                    try {
                        result = execute(cms, mqTopic, "edit");
                        if (result.isSuccess()) {
                            topic.setPartitions(mqTopic.getPartitions());
                            topic.setGmtModify(new Date());
                            cmsTopicRepository.update(topic);
                        }
                    } catch (Exception e) {
                        logger.error("create kafka topic partition failure:", e);
                    }
                    resourceEventPublisher.send(
                            new OperateResult(result.isSuccess(), "新增topic分区" + (result.isSuccess() ? "成功" : "失败"),
                                    "resource.kafka", null, OperateResult.OperateCategory.ResourceEvent, context));
                    CmsLog log = new CmsLog("消息队列", "新增topic分区",
                            JSONObject.toJSONString(new BocloudResult(result.isSuccess(),
                                    "新增topic分区" + (result.isSuccess() ? "成功" : "失败"))),
                            null, requestIp, new Date(), System.currentTimeMillis() - start,
                            id, context.getTarget());
                    cmsService.saveLog(log);
                });
                return new BocloudResult(true, "新增topic分区任务已下发...");
            }
            return new BocloudResult(false, "暂不支持该类操作");
        } catch (Exception e) {
            logger.error("create kafka topic partition failure:", e);
            return new BocloudResult(false, "新增topic分区失败");
        }
    }
}
