package com.bocloud.pms.service;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Map;
import java.util.concurrent.TimeUnit;

import org.apache.commons.lang3.StringUtils;
import org.bson.Document;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.Assert;

import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.GridBean;
import com.bocloud.common.model.Pager;
import com.bocloud.common.model.Param;
import com.bocloud.common.model.RequestContext;
import com.bocloud.common.utils.Common;
import com.bocloud.common.utils.GridHelper;
import com.bocloud.common.utils.ListTool;
import com.bocloud.pms.entity.Kvs;
import com.bocloud.pms.entity.KvsUser;
import com.bocloud.pms.entity.resource.enums.ResourceCategory;
import com.bocloud.pms.interfaces.middleware.KvsUserService;
import com.bocloud.pms.repository.KvsRepository;
import com.bocloud.pms.repository.KvsUserRepository;
import com.bocloud.pms.service.utils.MDSUtils;
import com.bocloud.service.lock.BocloudACLock;
import com.bocloud.service.lock.LockFactory;
import com.google.common.collect.Maps;
import com.mongodb.BasicDBObject;
import com.mongodb.MongoClient;
import com.mongodb.MongoClientOptions;
import com.mongodb.MongoCredential;
import com.mongodb.ReadPreference;
import com.mongodb.ServerAddress;
import com.mongodb.client.MongoDatabase;

@Service("kvsUserService")
public class KvsUserServiceImpl implements KvsUserService {

    private static Logger logger = LoggerFactory.getLogger(KvsUserServiceImpl.class);
    @Autowired
    private KvsRepository kvsRepository;
    @Autowired
    private KvsUserRepository kvsUserRepository;
    @Autowired
    private LockFactory lockFactory;

    @Override
    public BocloudResult<GridBean<KvsUser>> list(Pager pager, RequestContext context) {
        try {
            List<Param> params = pager.getParams();
            Integer page = pager.getPage();
            Integer rows = pager.getRows();
            Map<String, String> sorter = pager.getSorter();
            GridBean gridBean = null;
            if (null == params) {
                params = new ArrayList<>();
            }
            if (null == sorter) {
                sorter = Maps.newHashMap();
            }
            sorter.put("gmtCreate", Common.ONE);
            List<KvsUser> list = kvsUserRepository.list(page, rows, params, sorter);
            int total = kvsUserRepository.count(params);
            gridBean = GridHelper.getBean(page, rows, total, list);
            return new BocloudResult<>(true, gridBean, "查询成功");
        } catch (Exception e) {
            logger.error("list kvs failure:", e);
            return new BocloudResult<>(false, "查询失败");
        }
    }

    @Override
    public BocloudResult create(KvsUser kvsUser, RequestContext context) {
        Kvs kvs = null;
        try {
            kvs = kvsRepository.query(kvsUser.getKvsId());
            if (null == kvs) {
                logger.warn("kvs not exist");
                return new BocloudResult(false, "数据库不存在");
            }
            BocloudResult result = null;
            if (ResourceCategory.MongoDB.name().equalsIgnoreCase(kvs.getType())) {
                kvsUser.setPassword(MDSUtils.decrypt(kvsUser.getPassword()));
                result = executeMongod(kvs, kvsUser, "create");
                if (result.isFailed()) {
                    return new BocloudResult(false, kvs.getId(), "创建用户失败");
                }
                kvsUser.setPassword(kvsUser.getPassword());
                kvsUserRepository.save(kvsUser);
                return new BocloudResult(true, kvs.getId(), "创建用户成功");
            }
            return new BocloudResult(false, kvs.getId(), "暂不支持该类操作");
        } catch (Exception e) {
            logger.error("create kvs user failure:", e);
            return new BocloudResult(false, null == kvs ? null : kvs.getId(), "创建用户失败");
        }
    }

    @Override
    public BocloudResult remove(Long id, RequestContext context) {
        String path = KvsUser.class.getSimpleName() + "_" + id;
        Kvs kvs = null;
        try (BocloudACLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            KvsUser kvsUser = kvsUserRepository.query(id);
            if (null == kvsUser) {
                logger.warn("kvs user not exist");
                return new BocloudResult(false, "用户不存在");
            }
            kvs = kvsRepository.query(kvsUser.getKvsId());
            if (null == kvs) {
                logger.warn("kvs not exist");
                return new BocloudResult(false, "数据库不存在");
            }
            BocloudResult result = null;
            if (ResourceCategory.MongoDB.name().equalsIgnoreCase(kvs.getType())) {
                result = executeMongod(kvs, kvsUser, "remove");
                if (result.isFailed()) {
                    return new BocloudResult(false, kvs.getId(), "删除用户失败");
                }
                kvsUserRepository.remove(id, context.getTarget());
                return new BocloudResult(true, kvs.getId(), "删除用户成功");
            }
            return new BocloudResult(false, kvs.getId(), "暂不支持该类操作");
        } catch (Exception e) {
            logger.error("create kvs user failure:", e);
            return new BocloudResult(false, null == kvs ? null : kvs.getId(), "删除用户失败");
        }
    }

    @Override
    public BocloudResult<KvsUser> detail(Long id) {
        try {
            KvsUser user = kvsUserRepository.query(id);
            if (null == user) {
                logger.warn("kvs user not exist");
                return new BocloudResult<>(false, "数据不存在");
            }
            return new BocloudResult<>(true, user, "查询详情成功");
        } catch (Exception e) {
            logger.error("query kvs user detail fail:", e);
            return new BocloudResult<>(false, "查询详情失败");
        }
    }

    @Override
    public BocloudResult reset(KvsUser kvsUser, RequestContext context) {
        String path = KvsUser.class.getSimpleName() + "_" + kvsUser.getId();
        Kvs kvs = null;
        try (BocloudACLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            KvsUser object = kvsUserRepository.query(kvsUser.getId());
            if (null == object) {
                logger.warn("kvs user not exist");
                return new BocloudResult(false, "用户不存在");
            }
            kvs = kvsRepository.query(object.getKvsId());
            if (null == kvs) {
                logger.warn("kvs not exist");
                return new BocloudResult(false, "数据库不存在");
            }
            if (!object.getPassword().equals(kvsUser.getPassword())) {
                return new BocloudResult(false, kvs.getId(), "原密码输入错误");
            }
            if ("STOPPED".equals(kvs.getStatus())) {
                return new BocloudResult(false, kvs.getId(), "不是运行状态");
            }
            BocloudResult result = null;
            if (ResourceCategory.MongoDB.name().equalsIgnoreCase(kvs.getType())) {
                object.setNewPassword(MDSUtils.decrypt(kvsUser.getNewPassword()));
                result = executeMongod(kvs, object, "reset");
                if (result.isFailed()) {
                    return new BocloudResult(false, kvs.getId(), "重置用户密码失败");
                }
                object.setPassword(kvsUser.getNewPassword());
                kvsUserRepository.update(object);
                return new BocloudResult(true, kvs.getId(), "重置用户密码成功");
            }
            return new BocloudResult(false, kvs.getId(), "暂不支持该类操作");
        } catch (Exception e) {
            logger.error("reset kvs user failure:", e);
            return new BocloudResult(false, null == kvs ? null : kvs.getId(), "用户密码重置失败");
        }
    }

    private BocloudResult executeMongod(Kvs kvs, KvsUser user, String action) {
        MongoClient client = null;
        try {
//            int port = "cluster".equals(kvs.getCategory()) ? 27017 : 27017;
            int port = 27017;
            MongoCredential credential = MongoCredential.createScramSha1Credential(kvs.getUsername(), "admin",
                MDSUtils.decrypt(kvs.getPassword()).toCharArray());
            MongoClientOptions.Builder builder = MongoClientOptions.builder();
            builder.readPreference(ReadPreference.secondaryPreferred()).socketTimeout(5000).maxWaitTime(5000)
                .connectTimeout(5000).serverSelectionTimeout(5000);
            MongoClientOptions options = builder.build();
            client = new MongoClient(new ServerAddress(kvs.getManageIp(), port), credential, options);
            String dbName = StringUtils.isEmpty(user.getDbName()) ? "admin" : user.getDbName();
            MongoDatabase database = client.getDatabase(dbName);
            BasicDBObject command = null;
            switch (action) {
                case "create":
                    command = new BasicDBObject("createUser", user.getUsername()).append("pwd", user.getPassword())
                        .append("roles", Collections.singletonList(
                            new BasicDBObject("role", user.getPermission()).append("db", user.getDbName())));
                    break;
                case "remove":
                    BasicDBObject getUsersInfoCommand = new BasicDBObject("usersInfo",
                        new BasicDBObject("user", user.getUsername()).append("db", user.getDbName()));
                    Document result = database.runCommand(getUsersInfoCommand);
                    ArrayList<?> users = (ArrayList<?>)result.get("users");
                    if (ListTool.isEmpty(users)) {
                        return new BocloudResult(true, "该用户已不存在");
                    }
                    command = new BasicDBObject("dropUser", user.getUsername());
                    break;
                case "reset":
                    command = new BasicDBObject("updateUser", user.getUsername()).append("pwd", user.getNewPassword());
                    break;
                default:
                    return new BocloudResult(false, "暂不支持该类操作");
            }
            Document result = database.runCommand(command);
            if (null != result && result.get("ok").equals(1.0)) {
                return new BocloudResult(true, "操作成功");
            }
            return new BocloudResult(false, "操作失败");
        } catch (Exception e) {
            logger.info("operate mongodb user error:", e);
            return new BocloudResult(false, "数据库操作失败");
        } finally {
            if (null != client) {
                client.close();
            }
        }
    }

}
