package com.bocloud.pms.service;

import com.bocloud.common.model.*;
import com.bocloud.common.utils.Common;
import com.bocloud.common.utils.GridHelper;
import com.bocloud.pms.entity.Rds;
import com.bocloud.pms.entity.RdsDb;
import com.bocloud.pms.entity.RdsNode;
import com.bocloud.pms.entity.resource.enums.BocloudStatus;
import com.bocloud.pms.entity.resource.enums.ResourceCategory;
import com.bocloud.pms.interfaces.middleware.RdsDBService;
import com.bocloud.pms.repository.RdsDbRepository;
import com.bocloud.pms.repository.RdsNodeRepository;
import com.bocloud.pms.repository.RdsRepository;
import com.bocloud.pms.service.utils.MDSUtils;
import com.bocloud.service.lock.BocloudACLock;
import com.bocloud.service.lock.LockFactory;
import com.google.common.collect.Maps;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.Assert;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.concurrent.TimeUnit;

@Service("rdsDBService")
public class RdsDBServiceImpl implements RdsDBService {

    private static Logger logger = LoggerFactory.getLogger(RdsDBServiceImpl.class);
    @Autowired
    private RdsDbRepository rdsDbRepository;
    @Autowired
    private RdsRepository rdsRepository;
    @Autowired
    private LockFactory lockFactory;
    @Autowired
    private RdsNodeRepository rdsNodeRepository;

    @Override
    public BocloudResult<GridBean<RdsDb>> list(Pager pager, RequestContext context) {
        try {
            List<Param> params = pager.getParams();
            Integer page = pager.getPage();
            Integer rows = pager.getRows();
            Map<String, String> sorter = pager.getSorter();
            GridBean gridBean = null;
            if (null == params) {
                params = new ArrayList<>();
            }
            if (null == sorter) {
                sorter = Maps.newHashMap();
            }
            sorter.put("gmtCreate", Common.ONE);
            List<RdsDb> list = rdsDbRepository.list(page, rows, params, sorter);
            int total = rdsDbRepository.count(params);
            gridBean = GridHelper.getBean(page, rows, total, list);
            return new BocloudResult<>(true, gridBean, "查询成功");
        } catch (Exception e) {
            logger.error("list rds failure:", e);
            return new BocloudResult<>(false, "查询失败");
        }
    }

    @Override
    public BocloudResult create(RdsDb db, RequestContext context) {
        Rds rds = null;
        try {
            rds = rdsRepository.query(db.getRdsId());
            if (null == rds) {
                logger.warn("rds not exist");
                return new BocloudResult(false, "数据库不存在");
            }
            if ("STOPPED".equals(rds.getStatus())) {
                return new BocloudResult(false, rds.getId(), "数据库不是运行状态");
            }
            BocloudResult result = null;
            if (ResourceCategory.PostgreSQL.name().equalsIgnoreCase(rds.getType())) {
                String sql = "create database " + db.getName() + " encoding = '" + db.getCharacterSet()
                        + "' lc_collate = '" + db.getCollation() + "' lc_ctype = '" + db.getCollctype() + "';";
                result = executePostgresql(sql, rds);
            } else if (ResourceCategory.SqlServer.name().equalsIgnoreCase(rds.getType())) {
                String sql = "create database " + db.getName() + " collate " + db.getCollation() + ";";
                result = executeSqlServer(sql, rds);
            } else {
                if ("cluster".equals(rds.getCategory())) {
                    List<RdsNode> rdsNodes = rdsNodeRepository.list(rds.getId());
                    boolean checkCluster = MDSUtils.checkCluster(rdsNodes);
                    if (!checkCluster) {
                        return new BocloudResult(false, "集群结构不存在！请检查！");
                    }
                }
                String sql = "create database " + db.getName() + " character set " + db.getCharacterSet() + " collate "
                        + db.getCollation() + ";";
                result = executeMysql(sql, rds);
            }
            if (result.isFailed()) {
                return new BocloudResult(false, rds.getId(), "创建数据库失败");
            }
            db.setStatus(BocloudStatus.Basic.NORMAL.name());
            db.setCreatorId(context.getTarget());
            db.setGmtCreate(new Date());
            rdsDbRepository.save(db);
            return new BocloudResult(true, rds.getId(), "创建数据库成功");
        } catch (Exception e) {
            logger.error("create database error:", e);
            return new BocloudResult(false, null == rds ? null : rds.getId(), "创建数据库失败");
        }
    }

    @Override
    public BocloudResult remove(Long id, RequestContext context) {
        String path = RdsDb.class.getSimpleName() + "_" + id;
        Rds rds = null;
        try (BocloudACLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            RdsDb db = rdsDbRepository.query(id);
            if (null == db) {
                logger.warn("rds db not exist");
                return new BocloudResult(false, "数据库实例不存在");
            }
            rds = rdsRepository.query(db.getRdsId());
            if (null == rds) {
                logger.warn("rds not exist");
                return new BocloudResult(false, "数据库不存在");
            }
            if ("STOPPED".equals(rds.getStatus())) {
                return new BocloudResult(false, rds.getId(), "数据库不是运行状态");
            }
            String sql = "drop database if exists " + db.getName() + ";";
            BocloudResult result = null;
            if (ResourceCategory.PostgreSQL.name().equalsIgnoreCase(rds.getType())) {
                result = executePostgresql(sql, rds);
            } else if (ResourceCategory.SqlServer.name().equalsIgnoreCase(rds.getType())) {
                result = executeSqlServer(sql, rds);
            } else {
                if ("cluster".equals(rds.getCategory())) {
                    List<RdsNode> rdsNodes = rdsNodeRepository.list(rds.getId());
                    boolean checkCluster = MDSUtils.checkCluster(rdsNodes);
                    if (!checkCluster) {
                        return new BocloudResult(false, "集群结构不存在！请检查！");
                    }
                }
                result = executeMysql(sql, rds);
            }
            if (result.isFailed()) {
                if (null != result.getData()
                        && result.getData().toString().contains("is being accessed by other users")) {
                    return new BocloudResult(false, rds.getId(), "有用户正在访问该数据库");
                }
                return new BocloudResult(false, rds.getId(), "删除数据库失败");
            }
            rdsDbRepository.remove(id, context.getTarget());
            return new BocloudResult(true, rds.getId(), "删除数据库成功");
        } catch (Exception e) {
            logger.error("remove rds db failure:", e);
            return new BocloudResult(false, null == rds ? null : rds.getId(), "删除数据库失败");
        }
    }

    @Override
    public BocloudResult<RdsDb> detail(Long id) {
        try {
            RdsDb db = rdsDbRepository.query(id);
            if (null == db) {
                logger.warn("rds db not exist");
                return new BocloudResult<>(false, "数据不存在");
            }
            return new BocloudResult<>(true, db, "查询详情成功");
        } catch (Exception e) {
            logger.error("query rds db detail fail:", e);
            return new BocloudResult<>(false, "查询详情失败");
        }
    }

    private BocloudResult executeMysql(String sql, Rds rds) throws ClassNotFoundException {
        Class.forName("com.mysql.cj.jdbc.Driver");
        try (Connection conn = DriverManager.getConnection(
                "jdbc:mysql://" + rds.getManageIp()
                        + "/mysql?characterEncoding=utf-8&serverTimezone=UTC&useSSL=false",
                rds.getUsername(), MDSUtils.decrypt(rds.getPassword()))) {
            // 设置自动提交关闭
            conn.setAutoCommit(false);
            try (PreparedStatement pst = conn.prepareStatement(sql)) {
                pst.execute();
                conn.commit();
                return new BocloudResult(true, "操作数据库成功");
            } catch (SQLException e) {
                logger.info("operate database error:", e);
                return new BocloudResult(false, "操作数据库失败");
            }
        } catch (Exception e) {
            logger.info("operate database error:", e);
            return new BocloudResult(false, "操作数据库失败");
        }
    }

    private BocloudResult executePostgresql(String sql, Rds rds) throws ClassNotFoundException {
        Class.forName("org.postgresql.Driver");
        try (Connection conn = DriverManager.getConnection("jdbc:postgresql://" + rds.getManageIp() + ":5432/postgres",
                rds.getUsername(), MDSUtils.decrypt(rds.getPassword()));
             PreparedStatement pst = conn.prepareStatement(sql)) {
            pst.execute();
            return new BocloudResult(true, "操作数据库成功");
        } catch (Exception e) {
            logger.info("operate database error:", e);
            return new BocloudResult(false, e.getMessage(), "操作数据库失败");
        }
    }

    private BocloudResult executeSqlServer(String sql, Rds rds) throws ClassNotFoundException {
        Class.forName("com.microsoft.sqlserver.jdbc.SQLServerDriver");
        try (Connection conn = DriverManager.getConnection("jdbc:sqlserver://" + rds.getManageIp() + ":1433;database=master",
                rds.getUsername(), MDSUtils.decrypt(rds.getPassword()));
             PreparedStatement pst = conn.prepareStatement(sql)) {
            pst.execute();
            return new BocloudResult(true, "操作数据库成功");
        } catch (Exception e) {
            logger.info("operate database error:", e);
            return new BocloudResult(false, e.getMessage(), "操作数据库失败");
        }
    }

}
