package com.bocloud.pms.service.apply;

import java.util.ArrayList;
import java.util.List;

import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.amqp.core.AmqpTemplate;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.model.RequestContext;
import com.bocloud.common.model.Result;
import com.bocloud.pms.entity.resource.OrderItem;
import com.bocloud.pms.entity.resource.OrderMessage;
import com.bocloud.pms.entity.resource.ServiceOrder;
import com.bocloud.pms.entity.resource.enums.OrderItemStatus;
import com.bocloud.pms.entity.resource.enums.ResourceCategory;
import com.bocloud.pms.entity.resource.model.AlterationParam;
import com.bocloud.pms.entity.resource.model.ApplicationParam;
import com.bocloud.pms.entity.resource.model.CancelationParam;
import com.bocloud.pms.entity.resource.model.ExtensionParam;
import com.bocloud.pms.entity.resource.model.QuotaBean;
import com.bocloud.pms.interfaces.ResourceOperationService;
import com.bocloud.pms.interfaces.middleware.CmsService;
import com.bocloud.pms.interfaces.middleware.KvsService;
import com.bocloud.pms.interfaces.middleware.RdsService;

/**
 * 资源申请处理服务实现类
 */
@Service("resourceOperationService")
public class ResourceOperationServiceImpl implements ResourceOperationService {
    private static Logger logger = LoggerFactory.getLogger(ResourceOperationServiceImpl.class);
    @Autowired
    private AmqpTemplate amqpTemplate;
    @Autowired
    private RdsService rdsService;
    @Autowired
    private KvsService kvsService;
    @Autowired
    private CmsService cmsService;

    /**
     * 通知order 修改对应订单状态
     *
     * @param uuid
     * @param operation
     * @param message
     * @param status
     * @param context
     * @param newSn
     */
    private void syncOrderStatus(String uuid, OrderMessage.Operation operation, String message, OrderItemStatus status,
        RequestContext context, String newSn, List<QuotaBean> quotas, String category) {
        OrderMessage orderMessage =
            new OrderMessage(uuid, operation, status, null, message, null, null, new ArrayList<>(), category, context);
        orderMessage.setNewSn(newSn);
        orderMessage.setQuotas(quotas);
        amqpTemplate.convertAndSend("resource.result.queue", orderMessage);
    }

    @Override
    public Result process(ResourceCategory category, Object param, OrderMessage.Operation operation) {
        Result result = null;
        switch (operation) {
            case Application:
                result = application(category, param, operation);
                break;
            case Upgrade:
            case ScaleIn:
            case ScaleOut:
                result = alteration(category, param, operation);
                break;
            case Extension:
                result = extension(category, param, operation);
                break;
            case Cancelation:
                result = cancelation(category, param, operation);
                break;
            default:
                result = new Result(false, "暂不支持资源的其他类型操作");
                break;
        }
        return result;
    }

    /**
     * 服务申请
     *
     * @param category
     * @param param
     * @param context
     * @param operation
     * @return
     */
    private Result application(ResourceCategory category, Object object, OrderMessage.Operation operation) {
        ApplicationParam param = (ApplicationParam)object;
        BocloudResult result = null;
        try {
            switch (category) {
                case SqlServer:
                case PostgreSQL:
                case MySQL:
                    result = rdsService.apply(param, category, operation, param.getContext());
                    break;
                case MongoDB:
                case Redis:
                    result = kvsService.apply(param, category, operation, param.getContext());
                    break;
                case Kafka:
                case RabbitMQ:
                    result = cmsService.apply(param, category, operation, param.getContext());
                    break;
                default:
                    result = new BocloudResult(false, "暂不支持该类型资源申请");
                    break;
            }
            if (result.isFailed()) {
                if (!StringUtils.isEmpty(param.getCallback())) {
                    amqpTemplate.convertAndSend(param.getCallback(), JSONObject.toJSON(result));
                }
                ServiceOrder order = param.getOrder();
                List<OrderItem> items = order.getItems();
                List<QuotaBean> quotas = param.getQuotas();
                List<QuotaBean> beans = new ArrayList<>();
                for (QuotaBean bean : quotas) {
                    beans.add(new QuotaBean(bean.getCode(), bean.getValue() / items.size()));
                }
                for (OrderItem item : items) {
                    // 通知订单修改订单状态
                    syncOrderStatus(item.getSn(), operation, result.getMessage(), OrderItemStatus.OpenedFailure,
                        param.getContext(), null, beans, category.name());
                }
                return result;
            }
            return new BocloudResult(true, "资源申请处理成功");
        } catch (Exception e) {
            logger.error("Get error message:", e);
            if (!StringUtils.isEmpty(param.getCallback())) {
                amqpTemplate.convertAndSend(param.getCallback(),
                    JSONObject.toJSON(new BocloudResult(false, "资源申请处理失败：" + e.getMessage())));
            }
            return new BocloudResult(false, "资源申请处理失败！");
        }
    }

    /**
     * 服务变更
     *
     * @param category
     * @param param
     * @param context
     * @param operation
     * @return
     */
    private Result alteration(ResourceCategory category, Object object, OrderMessage.Operation operation) {
        AlterationParam param = (AlterationParam)object;
        BocloudResult result = null;
        try {
            switch (category) {
                case SqlServer:
                case PostgreSQL:
                case MySQL:
                    if (OrderMessage.Operation.Upgrade == operation) {
                        result = rdsService.apply(param, category, OrderMessage.Operation.Upgrade, param.getContext());
                    }
                    break;
                case Redis:
                case MongoDB:
                    if (OrderMessage.Operation.Upgrade == operation) {
                        result = kvsService.apply(param, category, OrderMessage.Operation.Upgrade, param.getContext());
                    }
                    break;
                case Kafka:
                case RabbitMQ:
                    if (OrderMessage.Operation.Upgrade == operation) {
                        result = cmsService.apply(param, category, OrderMessage.Operation.Upgrade, param.getContext());
                    }
                    break;
                default:
                    result = new BocloudResult(false, "暂不支持该类型资源变更");
                    break;
            }
            if (null != result && result.isFailed()) {
                // 通知订单修改订单状态
                syncOrderStatus(param.getInstance(), operation, result.getMessage(), OrderItemStatus.ChangeFailure,
                    param.getContext(), param.getNewInstance(), param.getQuotas(), category.name());
                return result;
            }
            return new BocloudResult(true, "资源变更成功处理成功");
        } catch (Exception e) {
            logger.error("Get error message:", e);
            return new BocloudResult(false, "资源变更处理失败！");
        }
    }

    /**
     * 服务延期
     *
     * @param category
     * @param param
     * @param context
     * @param operation
     * @return
     */
    private Result extension(ResourceCategory category, Object object, OrderMessage.Operation operation) {
        BocloudResult result = null;
        ExtensionParam param = (ExtensionParam)object;
        try {
            switch (category) {
                case PostgreSQL:
                case SqlServer:
                case MySQL:
                    result = rdsService.apply(param, category, operation, param.getContext());
                    break;
                case Redis:
                case MongoDB:
                    result = kvsService.apply(param, category, operation, param.getContext());
                    break;
                case Kafka:
                case RabbitMQ:
                    result = cmsService.apply(param, category, operation, param.getContext());
                    break;
                default:
                    result = new BocloudResult(false, "暂不支持该类型资源延期");
            }
            if (null != result && result.isFailed()) {
                // 通知订单修改订单状态
                syncOrderStatus(param.getInstance(), operation, result.getMessage(), OrderItemStatus.ExtensionFailure,
                    param.getContext(), null, null, category.name());
                // 通知订单修改订单状态
                return result;
            }
            return new BocloudResult(true, "资源变更成功处理成功");
        } catch (Exception e) {
            logger.error("Get error message:", e);
            return new BocloudResult(false, "资源变更处理失败！");
        }

    }

    /**
     * 服务退订
     *
     * @param category
     * @param param
     * @param context
     * @param operation
     * @return
     */
    private Result cancelation(ResourceCategory category, Object object, OrderMessage.Operation operation) {
        BocloudResult result = null;
        CancelationParam param = (CancelationParam)object;
        try {
            switch (category) {
                case SqlServer:
                case PostgreSQL:
                case MySQL:
                    result = rdsService.apply(param, category, operation, param.getContext());
                    break;
                case Redis:
                case MongoDB:
                    result = kvsService.apply(param, category, operation, param.getContext());
                    break;
                case Kafka:
                case RabbitMQ:
                    result = cmsService.apply(param, category, operation, param.getContext());
                    break;
                default:
                    result = new BocloudResult(false, "暂不支持该类型资源退订");
                    break;
            }
            if (null != result && result.isFailed()) {
                // 通知订单修改订单状态
                syncOrderStatus(param.getInstance(), operation, result.getMessage(), param.getOldStatus(),
                    param.getContext(), null, null, category.name());
                return result;
            }
            return new BocloudResult(true, "资源退订处理成功");
        } catch (Exception e) {
            logger.error("Get error message:", e);
            return new BocloudResult(false, "资源退订处理失败！");
        }
    }

}
